# prettyrepo
A wrapper for common [invoke](http://docs.pyinvoke.org/en/stable/) tasks to make repos extra pretty.


[![pipeline status](https://gitlab.com/tkutcher/prettyrepo/badges/dev/pipeline.svg)](https://gitlab.com/tkutcher/prettyrepo/-/commits/dev)
[![PyPI version](https://badge.fury.io/py/prettyrepo.svg)](https://badge.fury.io/py/prettyrepo)
[![coverage report](https://gitlab.com/tkutcher/prettyrepo/badges/dev/coverage.svg)](https://gitlab.com/tkutcher/prettyrepo/-/commits/dev)
[![docs](https://img.shields.io/static/v1?label=docs&message=wiki&color=purple)](https://gitlab.com/tkutcher/prettyrepo/-/wikis/home)


## Overview
All of the functionality as [invoke](http://docs.pyinvoke.org/en/stable/) with a smidge more flexibility (maybe just for me?) than something like [invocations](https://github.com/pyinvoke/invocations). 

In general, this project (as well as invoke), give an OS-agnostic solution for automating common commands during development (cleaning build files, running unit tests, etc.). Similar to how a Makefile might be used, prettyrepo makes it so that with a small set of commands you can run any typical task. Furthermore, you can have configurable arguments for each of those tasks being run (see [example](#example)).  

This project introduces a `TaskManager` so every available task can be tweaked as desired and persistent for a project. Reference [tasks.py](tasks.py) for this project's own tasks file.


### Example

```python

from invoke import task

from prettyrepo import TaskManager
from prettyrepo.utils import run_unittest
from prettyrepo.utils import get_python_cleans

# Create a TaskManager that knows the root of your project
mgr = TaskManager(source_root='prettyrepo', tasks_module=__name__)

# Add tasks from the "task library" or a simple command (by specifying the cmd kwarg)
mgr.add_task('clean', cleans=get_python_cleans())
mgr.add_task('test', lib='unittest', dir_='tests')
mgr.add_task('cov', aliases=['cover'], lib='coverage', func=run_unittest, dir_='tests')
mgr.add_task('lint', aliases=['flake8'], cmd='flake8 prettyrepo', doc='Run flake8 lints.')

# You can use the normal invoke task decorator in conjunction with prettyrepo tasks.
@task(name='bleh', help={'times': 'number of times to say blah'})
def blah(_, times=1):
    """Say blah."""
    print('\n'.join(['blah']*times))


ns = mgr.namespace
```

With this in your `tasks.py` file in the root of your repository, you can run 

`inv <command> [-h] [<command options...>]`

Where `-h` is available to show the help message. For example, running `inv clean -h` yields:

```
Docstring:
  Clean unnecessary files.

Options:
  -l INT, --level=INT       the level of "cleanliness" [0-2], default 1
  -v INT, --verbosity=INT   how verbose information to log to the console [0-3], default 1
```

Where `-l` and `-v` can be used to specify what the clean task does (the higher the level, the more files it removes - i.e. venv, dist files; the higher the verbosity, the more info it logs):


<img src="https://gitlab.com/tkutcher/prettyrepo/-/raw/dev/gfx/clean-demo.gif" alt="command line screenshot" height="400px">

 
 Documentation available in the [repository wiki](https://gitlab.com/tkutcher/prettyrepo/-/wikis/home/documentation/setup).
