
import os
import glob
import shutil
import itertools

from invoke import task

from prettyrepo import plog


PY_CLEANS = [
    ['./**/__pycache__', './**/*.pyc', './**/*.egg-info', './**/*.egg', '.coverage', 'coverage'],
    ['dist', 'build'],
    ['venv']
]


def get_task(mgr=None, task_name='clean', cleans=list(list()), run_from=None):
    return _clean_task_factory(
        root=mgr.project_root,
        logger=mgr.logger,
        task_name=task_name,
        cleans=cleans,
        run_from=run_from
    )


def clean(cleans, level=1, verbosity=1, logger=plog.QuietLogger()):

    def vlog(message, min_verbosity=1, color='yellow'):
        if verbosity >= min_verbosity:
            logger.log(message, level=plog.ALWAYS, color=color)

    vlog('cleaning...', 1)

    for clean_pattern in list(itertools.chain(*cleans[:level])):
        vlog(f'removing {clean_pattern}...', 2)

        for to_delete in glob.glob(clean_pattern, recursive=True):
            if not os.path.exists(to_delete):  # pragma: no cover
                continue
            try:
                if os.path.isfile(to_delete):
                    os.remove(to_delete)
                elif os.path.isdir(to_delete):
                    shutil.rmtree(to_delete)
                else:  # pragma: no cover
                    raise AssertionError('should be file or dir.')

                vlog(f'removed {to_delete}.', 3)

            except IOError:  # pragma: no cover
                vlog(f'could not delete {to_delete}', 0, color='red')

    vlog('done!', 1, color='green')


def _clean_task_factory(root, logger=plog.QuietLogger(), task_name='clean', cleans=list(list()),
                        run_from=None):

    if run_from is None:
        run_from = root

    @task(
        name=task_name,
        help={
            'level': f'the level of "cleanliness" [0-{len(cleans) - 1}], default 1',
            'verbosity': f'how verbose information to log to the console [0-3], default 1 '
        }
    )
    def clean_task(ctx, level=1, verbosity=1):
        """Clean unnecessary files."""

        with ctx.cd(run_from):  # pragma: no cover
            clean(cleans, level=level, verbosity=verbosity, logger=logger)

    return clean_task
