import os
import importlib

from invoke import task


def get_task(mgr, task_name='cover', func=None, include=None, omit=None, report_dir='htmlcov',
             run_from=os.getcwd(), *args, **kwargs):
    if include is None:
        include = f'{mgr.source_root}/*'
    return _coverage_task_factory(
        mgr,
        task_name=task_name,
        func=func,
        include=include,
        omit=omit,
        report_dir=report_dir,
        run_from=run_from,
        *args, **kwargs
    )


def _coverage_task_factory(mgr, task_name='cover', func=None, include=None, omit=None,
                           report_dir='coverage', run_from=os.getcwd(), *args, **kwargs):

    @task(
        name=task_name,
        help={
            'verbosity': f'how verbose information to log to the console [0-3], default 1 '
        }
    )
    def coverage_task(ctx, report='txt', branch=False):  # pragma: no cover
        """Run tests with coverage."""

        with ctx.cd(run_from):

            coverage = importlib.import_module('coverage')

            cov = coverage.coverage(
                branch=branch,
                include=include,
                omit=[] if omit is None else omit
            )

            cov.start()
            try:
                result = func(*args, **kwargs)
            except SystemExit:
                result = False
            cov.stop()
            cov.save()

            if report == 'txt':
                cov.report()
            elif report == 'html':
                if not os.path.exists(report_dir):
                    os.mkdir(report_dir)
                cov.html_report(directory=report_dir)
                mgr.logger.info(f'coverage report written to ./{report_dir}/')

            if not result:
                raise SystemExit(1)
            return result

    return coverage_task
