import os

from invoke import task

from prettyrepo import plog


_DEFAULT_TASK_NAME = 'flake8'


def get_task(mgr=None, task_name=_DEFAULT_TASK_NAME, dir_=None, run_from=os.getcwd()):
    if dir_ is None:
        dir_ = mgr.source_root
    return _flake8_task_factory(
        logger=mgr.logger if mgr else plog.QuietLogger(),
        task_name=task_name,
        dir_=dir_,
        run_from=run_from
    )


def _flake8_task_factory(logger=plog.QuietLogger(), task_name=_DEFAULT_TASK_NAME, dir_='src',
                         run_from=os.getcwd()):
    """Flake8 API is eh, so just going to use command line invocation for this one..."""

    @task(
        name=task_name,
        help={
            'verbose': 'boolean flag to be verbose at all',
            'verbosity': 'how verbose to be (default 0)'
        }
    )
    def flake8_task(ctx, verbose=False, verbosity=0):
        """Run flake8."""

        with ctx.cd(run_from):

            logger.log('running flake8...', color='yellow')

            result = ctx.run(
                f'flake8 '
                f'{("-" + ("v"*verbosity) + " ") if verbosity > 0 else ""}'
                f'{"-v " if verbose else ""}'
                f'{dir_}',
                echo=True
            )

        if result.ok:
            logger.info('all checks passed!')

        return result

    return flake8_task
