import os
import unittest

from invoke import task

from prettyrepo.plog import QuietLogger


def get_task(mgr=None, task_name='test', dir_='test', default_verbosity=1, run_from=os.getcwd()):
    logger = mgr.logger if mgr else QuietLogger()
    return _test_task_factory(
        logger,
        task_name=task_name,
        dir_=dir_,
        default_verbosity=default_verbosity,
        run_from=run_from
    )


def run_unittest(dir_, verbosity=1, can_exit=True):
    tests = unittest.TestLoader().discover(dir_)
    result = unittest.TextTestRunner(verbosity=verbosity).run(tests)
    if can_exit and not result.wasSuccessful():
        raise SystemExit(1)
    return result.wasSuccessful()


def _test_task_factory(
        logger=QuietLogger(),
        task_name='test',
        dir_='test',
        default_verbosity=1,
        run_from=os.getcwd()):

    @task(
        name=task_name,
        help={
            'verbosity': 'how verbose information to log to the console [0-3], default 1 ',
            'exitable': 'if included, means a SystemExit can be raised.'
        }
    )
    def test_task(ctx, verbosity=-1, exitable=False):
        """Run unittests."""

        with ctx.cd(run_from):

            verbosity = default_verbosity if verbosity == -1 else verbosity
            try:
                # get the result of the test run (which could raise an exit)
                result = run_unittest(dir_, verbosity=verbosity, can_exit=exitable)
                if not result:
                    raise SystemExit('dummy')
                logger.log('tests passed!', color='green')
            except SystemExit as e:
                logger.error('tests failed!')
                if exitable:
                    raise e

    return test_task
